import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import { css } from "@emotion/react";
import { getNamespaceFullName, getTypeName, } from "@typespec/compiler";
import React, { useContext } from "react";
import ReactDOMServer from "react-dom/server";
import { KeyValueSection, Literal } from "./common.js";
import { inspect } from "./inspect.js";
import { TypeUIBase } from "./type-ui-base.js";
import { getIdForType, isNamedUnion } from "./utils.js";
function expandNamespaces(namespace) {
    return [namespace, ...[...namespace.namespaces.values()].flatMap(expandNamespaces)];
}
const ProgramContext = React.createContext({});
export function renderProgram(program) {
    const html = ReactDOMServer.renderToString(_jsx(TypeSpecProgramViewer, { program: program }));
    return html;
}
const ProgramViewerStyles = css({
    fontFamily: "monospace",
    backgroundColor: "#f3f3f3",
    li: {
        margin: 0,
        listStyle: "none",
        position: "relative",
    },
});
export const TypeSpecProgramViewer = ({ program, }) => {
    const root = program.checker.getGlobalNamespaceType();
    const namespaces = expandNamespaces(root);
    return (_jsx(ProgramContext.Provider, { value: program, children: _jsx("div", { css: ProgramViewerStyles, children: _jsx("ul", { css: { padding: "0 0 0 10px", margin: 0 }, children: namespaces.map((namespace) => (_jsx("li", { children: _jsx(NamespaceUI, { type: namespace }) }, getNamespaceFullName(namespace)))) }) }) }));
};
export const ItemList = (props) => {
    if (Array.isArray(props.items)) {
        if (props.items.length === 0) {
            return _jsx(_Fragment, { children: "[]" });
        }
    }
    else {
        if (props.items.size === 0) {
            return _jsx(_Fragment, { children: "{}" });
        }
    }
    return (_jsx(KeyValueSection, { children: [...props.items.entries()].map(([k, v]) => (_jsx("li", { children: props.render(v) }, k))) }));
};
const omittedProps = [
    "kind",
    "name",
    "node",
    "symbol",
    "namespace",
    "templateNode",
    "templateArguments",
    "templateMapper",
    "instantiationParameters",
    "decorators",
    "projectionBase",
    "projectionsByName",
    "projectionSource",
    "projector",
    "projections",
    "isFinished",
];
const omittedPropsSet = new Set(omittedProps);
const NamedTypeUI = ({ type, name, properties }) => {
    name !== null && name !== void 0 ? name : (name = type.name);
    const propsUI = Object.entries(type)
        .map(([key, value]) => {
        if (omittedPropsSet.has(key)) {
            return undefined;
        }
        const action = properties[key];
        if (action === "skip") {
            return undefined;
        }
        const render = (x) => action === "ref" ? _jsx(TypeReference, { type: value }) : _jsx(TypeUI, { type: x });
        let valueUI;
        if (value === undefined) {
            valueUI = value;
        }
        else if (value.kind) {
            valueUI = render(value);
        }
        else if (typeof value === "object" &&
            "entries" in value &&
            typeof value.entries === "function") {
            valueUI = _jsx(ItemList, { items: value, render: render });
        }
        else {
            valueUI = value;
        }
        return {
            name: key,
            value: valueUI,
        };
    })
        .filter((x) => Boolean(x));
    return (_jsx(TypeUIBase, { type: type, name: name, properties: propsUI.concat([getDataProperty(type)]) }));
};
const TypeUI = ({ type }) => {
    switch (type.kind) {
        case "Namespace":
            return _jsx(NamespaceUI, { type: type });
        case "Interface":
            return _jsx(InterfaceUI, { type: type });
        case "Operation":
            return _jsx(OperationUI, { type: type });
        case "Model":
            return _jsx(ModelUI, { type: type });
        case "Scalar":
            return _jsx(ScalarUI, { type: type });
        case "ModelProperty":
            return _jsx(ModelPropertyUI, { type: type });
        case "Union":
            return _jsx(UnionUI, { type: type });
        case "UnionVariant":
            return _jsx(UnionVariantUI, { type: type });
        case "Enum":
            return _jsx(EnumUI, { type: type });
        case "EnumMember":
            return _jsx(EnumMemberUI, { type: type });
        default:
            return null;
    }
};
const NamespaceUI = ({ type }) => {
    const name = getNamespaceFullName(type) || "(global)";
    return (_jsx(NamedTypeUI, { name: name, type: type, properties: {
            namespaces: "skip",
            models: "nested",
            scalars: "nested",
            interfaces: "nested",
            operations: "nested",
            unions: "nested",
            enums: "nested",
            decoratorDeclarations: "nested",
            functionDeclarations: "nested",
        } }));
};
const InterfaceUI = ({ type }) => {
    return (_jsx(NamedTypeUI, { type: type, properties: {
            operations: "nested",
            sourceInterfaces: "ref",
        } }));
};
const OperationUI = ({ type }) => {
    return (_jsx(NamedTypeUI, { type: type, properties: {
            interface: "skip",
            parameters: "nested",
            returnType: "ref",
            sourceOperation: "ref",
        } }));
};
function getDataProperty(type) {
    return {
        name: "data",
        description: "in program.stateMap()",
        value: _jsx(TypeData, { type: type }),
    };
}
const ModelUI = ({ type }) => {
    return (_jsx(NamedTypeUI, { type: type, properties: {
            indexer: "skip",
            baseModel: "ref",
            derivedModels: "ref",
            properties: "nested",
            sourceModel: "ref",
        } }));
};
const ScalarUI = ({ type }) => {
    return (_jsx(NamedTypeUI, { type: type, properties: {
            baseScalar: "ref",
            derivedScalars: "ref",
        } }));
};
const ModelPropertyUI = ({ type }) => {
    return (_jsx(NamedTypeUI, { type: type, properties: {
            model: "skip",
            type: "ref",
            optional: "value",
            sourceProperty: "ref",
            default: "value",
        } }));
};
const EnumUI = ({ type }) => {
    return (_jsx(NamedTypeUI, { type: type, properties: {
            members: "nested",
        } }));
};
const EnumMemberUI = ({ type }) => {
    return (_jsx(NamedTypeUI, { type: type, properties: {
            enum: "skip",
            sourceMember: "ref",
            value: "value",
        } }));
};
const UnionUI = ({ type }) => {
    if (!isNamedUnion(type)) {
        return _jsx(_Fragment, {});
    }
    return (_jsx(NamedTypeUI, { type: type, properties: {
            expression: "skip",
            options: "skip",
            variants: "nested",
        } }));
};
const UnionVariantUI = ({ type }) => {
    if (typeof type.name === "symbol") {
        return _jsx(_Fragment, {});
    }
    return (_jsx(NamedTypeUI, { type: type, properties: {
            union: "skip",
            type: "ref",
        } }));
};
const NamedTypeRef = ({ type }) => {
    const id = getIdForType(type);
    const href = `#${id}`;
    return (_jsx("a", { css: {
            color: "#268bd2",
            textDecoration: "none",
            "&:hover": {
                textDecoration: "underline",
            },
        }, href: href, title: type.kind + ": " + id, children: getTypeName(type) }));
};
const TypeReference = ({ type }) => {
    switch (type.kind) {
        case "Namespace":
        case "Operation":
        case "Interface":
        case "Enum":
        case "ModelProperty":
        case "Scalar":
            return _jsx(NamedTypeRef, { type: type });
        case "Model":
            if (type.name === "") {
                return (_jsx(KeyValueSection, { children: _jsx(TypeUI, { type: type }) }));
            }
            else {
                return _jsx(NamedTypeRef, { type: type });
            }
        case "Union":
            if (isNamedUnion(type)) {
                return _jsx(NamedTypeRef, { type: type });
            }
            else {
                return (_jsx(_Fragment, { children: [...type.variants.values()].map((variant, i) => {
                        return (_jsxs("span", { children: [_jsx(TypeReference, { type: variant.type }), i < type.variants.size - 1 ? " | " : ""] }, i));
                    }) }));
            }
        case "TemplateParameter":
            return _jsxs("span", { children: ["Template Param: ", type.node.id.sv] });
        case "String":
            return _jsxs(Literal, { children: ["\"", type.value, "\""] });
        case "Number":
        case "Boolean":
            return _jsx(_Fragment, { children: type.value });
        default:
            return null;
    }
};
const TypeData = ({ type }) => {
    const program = useContext(ProgramContext);
    const entries = [...program.stateMaps.entries()]
        .map(([k, v]) => { var _a; return [k, (_a = v.get(undefined)) === null || _a === void 0 ? void 0 : _a.get(type)]; })
        .filter(([k, v]) => !!v);
    if (entries.length === 0) {
        return null;
    }
    return (_jsx(KeyValueSection, { children: entries.map(([k, v], i) => (_jsxs("div", { css: { display: "flex" }, children: [_jsxs("div", { css: { color: "#333", marginRight: "5px" }, children: [k.toString(), ":"] }), " ", _jsx("div", { children: inspect(v) })] }, i))) }));
};
//# sourceMappingURL=ui.js.map